@@ -1,6 +1,6 @@
 # -*- encoding: utf-8 -*-
 #
-# Copyright © 2016-2017 Red Hat, Inc.
+# Copyright © 2016-2018 Red Hat, Inc.
 # Copyright © 2014-2015 eNovance
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -17,10 +17,10 @@
 import collections
 import functools
 import itertools
+import operator
 import uuid
 
 import jsonpatch
-import pbr.version
 import pecan
 from pecan import rest
 import pyparsing
@@ -31,7 +31,10 @@ import tooz
 import voluptuous
 import werkzeug.http
 
+import gnocchi
 from gnocchi import archive_policy
+from gnocchi import calendar
+from gnocchi import chef
 from gnocchi.cli import metricd
 from gnocchi import incoming
 from gnocchi import indexer
@@ -50,6 +53,9 @@ except ImportError:
     PROMETHEUS_SUPPORTED = False
 
 
+ATTRGETTER_GRANULARITY = operator.attrgetter("granularity")
+
+
 def arg_to_list(value):
     if isinstance(value, list):
         return value
@@ -290,7 +296,7 @@ class ArchivePoliciesController(rest.RestController):
         enforce("create archive policy", {})
         # NOTE(jd): Initialize this one at run-time because we rely on conf
         conf = pecan.request.conf
-        valid_agg_methods = (
+        valid_agg_methods = list(
             archive_policy.ArchivePolicy.VALID_AGGREGATION_METHODS_VALUES
         )
         ArchivePolicySchema = voluptuous.Schema({
@@ -302,7 +308,7 @@ class ArchivePoliciesController(rest.RestController):
             voluptuous.Required(
                 "aggregation_methods",
                 default=list(conf.archive_policy.default_aggregation_methods)):
-            voluptuous.All(list(valid_agg_methods)),
+            valid_agg_methods,
             voluptuous.Required("definition"): ArchivePolicyDefinitionSchema,
         })
 
@@ -454,43 +460,24 @@ class MetricController(rest.RestController):
                      granularity=None, resample=None, refresh=False,
                      **param):
         self.enforce_metric("get measures")
-        if (aggregation not in
-           archive_policy.ArchivePolicy.VALID_AGGREGATION_METHODS):
-            msg = "Invalid aggregation value %(agg)s, must be one of %(std)s"
-            abort(400, msg % dict(
-                agg=aggregation,
-                std=archive_policy.ArchivePolicy.VALID_AGGREGATION_METHODS))
-
-        if start is not None:
-            try:
-                start = utils.to_timestamp(start)
-            except Exception:
-                abort(400, "Invalid value for start")
-
-        if stop is not None:
-            try:
-                stop = utils.to_timestamp(stop)
-            except Exception:
-                abort(400, "Invalid value for stop")
 
         if resample:
             if not granularity:
                 abort(400, 'A granularity must be specified to resample')
             try:
-                resample = utils.to_timespan(resample)
+                resample = (resample if calendar.GROUPINGS.get(resample) else
+                            utils.to_timespan(resample))
             except ValueError as e:
                 abort(400, six.text_type(e))
 
         if granularity is None:
             granularity = [d.granularity
                            for d in self.metric.archive_policy.definition]
+            start, stop, _, _, _ = validate_qs(
+                start=start, stop=stop)
         else:
-            try:
-                granularity = [utils.to_timespan(granularity)]
-            except ValueError:
-                abort(400, {"cause": "Attribute value error",
-                            "detail": "granularity",
-                            "reason": "Invalid granularity"})
+            start, stop, granularity, _, _ = validate_qs(
+                start=start, stop=stop, granularity=granularity)
 
         if aggregation not in self.metric.archive_policy.aggregation_methods:
             abort(404, {
@@ -501,21 +488,33 @@ class MetricController(rest.RestController):
                 },
             })
 
+        aggregations = []
+        for g in sorted(granularity, reverse=True):
+            agg = self.metric.archive_policy.get_aggregation(
+                aggregation, g)
+            if agg is None:
+                abort(404, six.text_type(
+                    storage.AggregationDoesNotExist(
+                        self.metric, aggregation, g)
+                ))
+            aggregations.append(agg)
+
         if (strtobool("refresh", refresh) and
                 pecan.request.incoming.has_unprocessed(self.metric.id)):
             try:
-                pecan.request.storage.refresh_metric(
-                    pecan.request.indexer, pecan.request.incoming, self.metric,
+                pecan.request.chef.refresh_metrics(
+                    [self.metric],
                     pecan.request.conf.api.operation_timeout)
-            except storage.SackLockTimeoutError as e:
-                abort(503, six.text_type(e))
+            except chef.SackAlreadyLocked:
+                abort(503, 'Unable to refresh metric: %s. Metric is locked. '
+                      'Please try again.' % self.metric.id)
         try:
             return pecan.request.storage.get_measures(
-                self.metric, granularity, start, stop, aggregation,
-                resample)
-        except (storage.MetricDoesNotExist,
-                storage.AggregationDoesNotExist) as e:
+                self.metric, aggregations, start, stop, resample)[aggregation]
+        except storage.AggregationDoesNotExist as e:
             abort(404, six.text_type(e))
+        except storage.MetricDoesNotExist:
+            return []
 
     @pecan.expose()
     def delete(self):
@@ -1271,7 +1270,7 @@ class QueryStringSearchAttrFilter(object):
         try:
             parsed_query = cls.expr.parseString(query, parseAll=True)[0]
         except pyparsing.ParseException as e:
-            raise abort(400, "Invalid filter: %s" % six.text_type(e))
+            raise abort(400, "Invalid filter: %s" % str(e))
         return cls._parsed_query2dict(parsed_query)
 
     @classmethod
@@ -1458,6 +1457,101 @@ class SearchMetricController(rest.RestController):
         )
     )
 
+    class MeasureQuery(object):
+        binary_operators = {
+            u"=": operator.eq,
+            u"==": operator.eq,
+            u"eq": operator.eq,
+
+            u"<": operator.lt,
+            u"lt": operator.lt,
+
+            u">": operator.gt,
+            u"gt": operator.gt,
+
+            u"<=": operator.le,
+            u"≤": operator.le,
+            u"le": operator.le,
+
+            u">=": operator.ge,
+            u"≥": operator.ge,
+            u"ge": operator.ge,
+
+            u"!=": operator.ne,
+            u"≠": operator.ne,
+            u"ne": operator.ne,
+
+            u"%": operator.mod,
+            u"mod": operator.mod,
+
+            u"+": operator.add,
+            u"add": operator.add,
+
+            u"-": operator.sub,
+            u"sub": operator.sub,
+
+            u"*": operator.mul,
+            u"×": operator.mul,
+            u"mul": operator.mul,
+
+            u"/": operator.truediv,
+            u"÷": operator.truediv,
+            u"div": operator.truediv,
+
+            u"**": operator.pow,
+            u"^": operator.pow,
+            u"pow": operator.pow,
+        }
+
+        multiple_operators = {
+            u"or": any,
+            u"∨": any,
+            u"and": all,
+            u"∧": all,
+        }
+
+        def __init__(self, tree):
+            self._eval = self.build_evaluator(tree)
+
+        def __call__(self, value):
+            return self._eval(value)
+
+        def build_evaluator(self, tree):
+            try:
+                operator, nodes = list(tree.items())[0]
+            except Exception:
+                return lambda value: tree
+            try:
+                op = self.multiple_operators[operator]
+            except KeyError:
+                try:
+                    op = self.binary_operators[operator]
+                except KeyError:
+                    raise self.InvalidQuery("Unknown operator %s" % operator)
+                return self._handle_binary_op(op, nodes)
+            return self._handle_multiple_op(op, nodes)
+
+        def _handle_multiple_op(self, op, nodes):
+            elements = [self.build_evaluator(node) for node in nodes]
+            return lambda value: op((e(value) for e in elements))
+
+        def _handle_binary_op(self, op, node):
+            try:
+                iterator = iter(node)
+            except Exception:
+                return lambda value: op(value, node)
+            nodes = list(iterator)
+            if len(nodes) != 2:
+                raise self.InvalidQuery(
+                    "Binary operator %s needs 2 arguments, %d given" %
+                    (op, len(nodes)))
+            node0 = self.build_evaluator(node[0])
+            node1 = self.build_evaluator(node[1])
+            return lambda value: op(node0(value), node1(value))
+
+        class InvalidQuery(Exception):
+            pass
+
     @pecan.expose('json')
     def post(self, metric_id, start=None, stop=None, aggregation='mean',
              granularity=None):
@@ -1485,8 +1579,8 @@ class SearchMetricController(rest.RestController):
                 abort(400, "Invalid value for stop")
 
         try:
-            predicate = storage.MeasureQuery(query)
-        except storage.InvalidQuery as e:
+            predicate = self.MeasureQuery(query)
+        except self.MeasureQuery.InvalidQuery as e:
             abort(400, six.text_type(e))
 
         if granularity is not None:
@@ -1494,26 +1588,40 @@ class SearchMetricController(rest.RestController):
                 map(utils.to_timespan, arg_to_list(granularity)),
                 reverse=True)
 
-        results = {}
+        metrics_and_aggregations = collections.defaultdict(list)
+
+        for metric in metrics:
+            if granularity is None:
+                granularity = sorted((
+                    d.granularity
+                    for d in metric.archive_policy.definition),
+                    reverse=True)
+            for gr in granularity:
+                agg = metric.archive_policy.get_aggregation(
+                    aggregation, gr)
+                if agg is None:
+                    abort(400,
+                          storage.AggregationDoesNotExist(
+                              metric, aggregation, gr))
+                metrics_and_aggregations[metric].append(agg)
 
         try:
-            for metric in metrics:
-                if granularity is None:
-                    granularity = sorted((
-                        d.granularity
-                        for d in metric.archive_policy.definition),
-                        reverse=True)
-                results[str(metric.id)] = []
-                for r in utils.parallel_map(
-                        pecan.request.storage.find_measure,
-                        ((metric, predicate, g, aggregation,
-                          start, stop)
-                         for g in granularity)):
-                    results[str(metric.id)].extend(r)
-        except storage.AggregationDoesNotExist as e:
+            timeseries = pecan.request.storage.get_aggregated_measures(
+                metrics_and_aggregations, start, stop)
+        except storage.MetricDoesNotExist as e:
+            # This can happen if all the metrics have been created but one
+            # doesn't have any measures yet.
             abort(400, e)
 
-        return results
+        return {
+            str(metric.id): [
+                (timestamp, aggregation.granularity, value)
+                for aggregation, ts in six.iteritems(aggregations_and_ts)
+                for timestamp, value in ts
+                if predicate(value)
+            ]
+            for metric, aggregations_and_ts in six.iteritems(timeseries)
+        }
 
 
 class ResourcesMetricsMeasuresBatchController(rest.RestController):
@@ -1552,9 +1660,14 @@ class ResourcesMetricsMeasuresBatchController(rest.RestController):
         attribute_filter = {"or": []}
         for original_resource_id, resource_id in body:
             names = list(body[(original_resource_id, resource_id)].keys())
-            attribute_filter["or"].append({"and": [
-                {"=": {"resource_id": resource_id}},
-                {"in": {"name": names}}]})
+            if names:
+                attribute_filter["or"].append({"and": [
+                    {"=": {"resource_id": resource_id}},
+                    {"in": {"name": names}}]})
+
+        if not attribute_filter["or"]:
+            pecan.response.status = 202
+            return
 
         all_metrics = collections.defaultdict(list)
         for metric in pecan.request.indexer.list_metrics(
@@ -1681,7 +1794,8 @@ class AggregationResourceController(rest.RestController):
     @pecan.expose('json')
     def post(self, start=None, stop=None, aggregation='mean',
              reaggregation=None, granularity=None, needed_overlap=100.0,
-             groupby=None, fill=None, refresh=False, resample=None):
+             groupby=None, fill=None, refresh=False, resample=None,
+             **kwargs):
         # First, set groupby in the right format: a sorted list of unique
         # strings.
         groupby = sorted(set(arg_to_list(groupby)))
@@ -1690,7 +1804,8 @@ class AggregationResourceController(rest.RestController):
         # groups when using itertools.groupby later.
         try:
             resources = SearchResourceTypeController(
-                self.resource_type)._search(sort=groupby)
+                self.resource_type)._search(sort=groupby,
+                                            filter=kwargs.get("filter"))
         except indexer.InvalidPagination:
             abort(400, "Invalid groupby attribute")
         except indexer.IndexerException as e:
@@ -1724,15 +1839,13 @@ class AggregationResourceController(rest.RestController):
 
         return results
 
-
 FillSchema = voluptuous.Schema(
     voluptuous.Any(voluptuous.Coerce(float), "null", "dropna",
                    msg="Must be a float, 'dropna' or 'null'"))
 
 
-# FIXME(sileht): should be in aggregates.api but we need to split all
-# controllers to do this
-def validate_qs(start, stop, granularity, needed_overlap, fill):
+def validate_qs(start=None, stop=None, granularity=None,
+                needed_overlap=None, fill=None):
     if needed_overlap is not None:
         try:
             needed_overlap = float(needed_overlap)
@@ -1805,14 +1918,6 @@ class AggregationController(rest.RestController):
         start, stop, granularity, needed_overlap, fill = validate_qs(
             start, stop, granularity, needed_overlap, fill)
 
-        if (aggregation
-           not in archive_policy.ArchivePolicy.VALID_AGGREGATION_METHODS):
-            abort(
-                400,
-                'Invalid aggregation value %s, must be one of %s'
-                % (aggregation,
-                   archive_policy.ArchivePolicy.VALID_AGGREGATION_METHODS))
-
         if reaggregation is None:
             reaggregation = aggregation
 
@@ -1827,7 +1932,8 @@ class AggregationController(rest.RestController):
             if not granularity:
                 abort(400, 'A granularity must be specified to resample')
             try:
-                resample = utils.to_timespan(resample)
+                resample = (resample if calendar.GROUPINGS.get(resample) else
+                            utils.to_timespan(resample))
             except ValueError as e:
                 abort(400, six.text_type(e))
 
@@ -1851,6 +1957,19 @@ class AggregationController(rest.RestController):
                          for metric in metrics),
                     'No granularity match'))
 
+        aggregations = set()
+        for metric in metrics:
+            for g in granularity:
+                agg = metric.archive_policy.get_aggregation(
+                    aggregation, g)
+                if agg is None:
+                    abort(404, six.text_type(
+                        storage.AggregationDoesNotExist(metric, aggregation, g)
+                    ))
+                aggregations.add(agg)
+        aggregations = sorted(aggregations, key=ATTRGETTER_GRANULARITY,
+                              reverse=True)
+
         operations = ["aggregate", reaggregation, []]
         if resample:
             operations[2].extend(
@@ -1871,11 +1990,12 @@ class AggregationController(rest.RestController):
                     if pecan.request.incoming.has_unprocessed(m.id)]
                 for m in metrics_to_update:
                     try:
-                        pecan.request.storage.refresh_metric(
-                            pecan.request.indexer, pecan.request.incoming, m,
-                            pecan.request.conf.api.operation_timeout)
-                    except storage.SackLockTimeoutError as e:
-                        abort(503, six.text_type(e))
+                        pecan.request.chef.refresh_metrics(
+                            [m], pecan.request.conf.api.operation_timeout)
+                    except chef.SackAlreadyLocked:
+                        abort(503, 'Unable to refresh metric: %s. '
+                              'Metric is locked. '
+                              'Please try again.' % m.id)
             if number_of_metrics == 1:
                 # NOTE(sileht): don't do the aggregation if we only have one
                 # metric
@@ -1890,9 +2010,12 @@ class AggregationController(rest.RestController):
                             "aggregation_method": aggregation,
                         },
                     })
-                return pecan.request.storage.get_measures(
-                    metric, granularity, start, stop, aggregation,
-                    resample)
+                try:
+                    return pecan.request.storage.get_measures(
+                        metric, aggregations, start, stop, resample
+                    )[aggregation]
+                except storage.MetricDoesNotExist:
+                    return []
             return processor.get_measures(
                 pecan.request.storage,
                 [processor.MetricReference(m, aggregation) for m in metrics],
@@ -1900,8 +2023,7 @@ class AggregationController(rest.RestController):
                 granularity, needed_overlap, fill)["aggregated"]
         except exceptions.UnAggregableTimeseries as e:
             abort(400, e)
-        except (storage.MetricDoesNotExist,
-                storage.AggregationDoesNotExist) as e:
+        except storage.AggregationDoesNotExist as e:
             abort(404, six.text_type(e))
 
     MetricIDsSchema = [utils.UUID]
@@ -1971,9 +2093,20 @@ class StatusController(rest.RestController):
             report_dict["storage"]["measures_to_process"] = report['details']
         report_dict['metricd'] = {}
         if members_req:
-            report_dict['metricd']['processors'] = members_req.get()
+            members = members_req.get()
+            caps = [
+                pecan.request.coordinator.get_member_capabilities(
+                    metricd.MetricProcessor.GROUP_ID, member)
+                for member in members
+            ]
+            report_dict['metricd']['processors'] = members
+            report_dict['metricd']['statistics'] = {
+                member: cap.get()
+                for member, cap in six.moves.zip(members, caps)
+            }
         else:
             report_dict['metricd']['processors'] = None
+            report_dict['metricd']['statistics'] = {}
         return report_dict
 
 
@@ -2178,7 +2311,7 @@ class VersionsController(object):
     @pecan.expose('json')
     def index():
         return {
-            "build": pbr.version.VersionInfo('gnocchi').version_string(),
+            "build": gnocchi.__version__,
             "versions": [
                 {
                     "status": "CURRENT",
