@@ -44,7 +44,7 @@ def datetime64(*args):
 class TestAggregatedTimeseries(base.BaseTestCase):
     @staticmethod
     def _resample_and_merge(ts, agg_dict):
-        """Helper method that mimics _add_measures workflow."""
+        """Helper method that mimics _compute_splits_operations workflow."""
         grouped = ts.group_serie(agg_dict['sampling'])
         existing = agg_dict.get('return')
         name = agg_dict.get("name")
@@ -53,7 +53,10 @@ class TestAggregatedTimeseries(base.BaseTestCase):
         agg_dict['return'] = (
             processor.MetricReference(metric, "mean", resource),
             carbonara.AggregatedTimeSerie.from_grouped_serie(
-                grouped, agg_dict['sampling'], agg_dict['agg']))
+                grouped,
+                carbonara.Aggregation(agg_dict['agg'],
+                                      agg_dict['sampling'],
+                                      None)))
         if existing:
             existing[2].merge(agg_dict['return'][2])
             agg_dict['return'] = existing
@@ -88,8 +91,7 @@ class TestAggregatedTimeseries(base.BaseTestCase):
                 'size': 50, 'agg': 'mean'}
         tsb1 = carbonara.BoundTimeSerie(block_size=tsc1['sampling'])
         tsc2 = carbonara.AggregatedTimeSerie(
-            sampling=numpy.timedelta64(60, 's'),
-            aggregation_method='mean')
+            carbonara.Aggregation('mean', numpy.timedelta64(60, 's'), None))
 
         tsb1.set_values(numpy.array([(datetime64(2014, 1, 1, 12, 3, 0), 4)],
                                     dtype=carbonara.TIMESERIES_ARRAY_DTYPE),
@@ -1029,7 +1031,7 @@ class CrossMetricAggregated(base.TestCase):
             incoming.Measure(datetime64(2014, 1, 1, 12, 10, 31), 4),
             incoming.Measure(datetime64(2014, 1, 1, 12, 13, 10), 4),
         ])
-        self.trigger_processing([str(self.metric.id), str(metric2.id)])
+        self.trigger_processing([self.metric, metric2])
 
         values = processor.get_measures(
             self.storage,
@@ -1188,7 +1190,7 @@ class CrossMetricAggregated(base.TestCase):
             incoming.Measure(datetime64(2014, 1, 1, 12, 9, 31), 6),
             incoming.Measure(datetime64(2014, 1, 1, 12, 13, 10), 2),
         ])
-        self.trigger_processing([str(self.metric.id), str(metric2.id)])
+        self.trigger_processing([self.metric, metric2])
 
         values = processor.get_measures(
             self.storage,
@@ -1226,7 +1228,7 @@ class CrossMetricAggregated(base.TestCase):
             incoming.Measure(datetime64(2014, 1, 1, 14, 2, 31), 4),
             incoming.Measure(datetime64(2014, 1, 1, 15, 3, 10), 4),
         ])
-        self.trigger_processing([str(self.metric.id), str(metric2.id)])
+        self.trigger_processing([self.metric, metric2])
 
         values = processor.get_measures(
             self.storage,
@@ -1263,7 +1265,7 @@ class CrossMetricAggregated(base.TestCase):
             incoming.Measure(datetime64(2014, 1, 1, 14, 2, 31), 4),
             incoming.Measure(datetime64(2014, 1, 1, 15, 3, 10), 4),
         ])
-        self.trigger_processing([str(self.metric.id), str(metric2.id)])
+        self.trigger_processing([self.metric, metric2])
 
         values = processor.get_measures(
             self.storage,
@@ -1300,7 +1302,7 @@ class CrossMetricAggregated(base.TestCase):
             incoming.Measure(datetime64(2014, 1, 1, 14, 2, 31), 4),
             incoming.Measure(datetime64(2014, 1, 1, 15, 3, 10), 4),
         ])
-        self.trigger_processing([str(self.metric.id), str(metric2.id)])
+        self.trigger_processing([self.metric, metric2])
 
         values = processor.get_measures(
             self.storage,
@@ -1339,7 +1341,7 @@ class CrossMetricAggregated(base.TestCase):
             incoming.Measure(datetime64(2014, 1, 1, 12, 10, 31), 4),
             incoming.Measure(datetime64(2014, 1, 1, 12, 15, 10), 4),
         ])
-        self.trigger_processing([str(self.metric.id), str(metric2.id)])
+        self.trigger_processing([self.metric, metric2])
 
         values = processor.get_measures(
             self.storage,
@@ -1383,7 +1385,7 @@ class CrossMetricAggregated(base.TestCase):
             incoming.Measure(datetime64(2014, 1, 1, 14, 2, 31), 4),
             incoming.Measure(datetime64(2014, 1, 1, 15, 3, 10), 4),
         ])
-        self.trigger_processing([str(self.metric.id), str(metric2.id)])
+        self.trigger_processing([self.metric, metric2])
 
         values = processor.get_measures(
             self.storage,
@@ -1412,7 +1414,7 @@ class CrossMetricAggregated(base.TestCase):
             incoming.Measure(datetime64(2014, 1, 1, 14, 2, 31), 4),
             incoming.Measure(datetime64(2014, 1, 1, 15, 3, 45), 44),
         ])
-        self.trigger_processing([str(self.metric.id)])
+        self.trigger_processing()
 
         values = processor.get_measures(
             self.storage, [processor.MetricReference(self.metric, "mean")],
@@ -1439,7 +1441,7 @@ class CrossMetricAggregated(base.TestCase):
             incoming.Measure(datetime64(2014, 1, 1, 14, 2, 31), 4),
             incoming.Measure(datetime64(2014, 1, 1, 15, 3, 45), 44),
         ])
-        self.trigger_processing([str(self.metric.id)])
+        self.trigger_processing()
 
         values = processor.get_measures(
             self.storage, [processor.MetricReference(self.metric, "mean")],
@@ -1471,7 +1473,7 @@ class CrossMetricAggregated(base.TestCase):
             incoming.Measure(datetime64(2014, 1, 1, 14, 2, 31), 4),
             incoming.Measure(datetime64(2014, 1, 1, 15, 3, 10), 4),
         ])
-        self.trigger_processing([str(self.metric.id), str(metric2.id)])
+        self.trigger_processing([self.metric, metric2])
 
         values = processor.get_measures(
             self.storage,
@@ -1509,7 +1511,7 @@ class CrossMetricAggregated(base.TestCase):
             incoming.Measure(datetime64(2014, 1, 1, 14, 2, 31), 4),
             incoming.Measure(datetime64(2014, 1, 1, 15, 3, 10), 4),
         ])
-        self.trigger_processing([str(self.metric.id), str(metric2.id)])
+        self.trigger_processing([self.metric, metric2])
 
         values = processor.get_measures(
             self.storage,
@@ -1555,7 +1557,7 @@ class CrossMetricAggregated(base.TestCase):
             incoming.Measure(datetime64(2014, 1, 1, 14, 2, 31), 4),
             incoming.Measure(datetime64(2014, 1, 1, 15, 3, 10), -4),
         ])
-        self.trigger_processing([str(self.metric.id), str(metric2.id)])
+        self.trigger_processing([self.metric, metric2])
 
         values = processor.get_measures(
             self.storage,
