@@ -27,17 +27,16 @@ import fixtures
 import iso8601
 from keystonemiddleware import fixture as ksm_fixture
 import mock
-import pbr.version
 import six
 import testscenarios
 from testtools import testcase
 import webtest
 
+import gnocchi
 from gnocchi import archive_policy
 from gnocchi.rest import api
 from gnocchi.rest import app
 from gnocchi.tests import base as tests_base
-from gnocchi.tests import utils as tests_utils
 from gnocchi import utils
 
 
@@ -61,9 +60,7 @@ class TestingApp(webtest.TestApp):
 
     def __init__(self, *args, **kwargs):
         self.auth_mode = kwargs.pop('auth_mode')
-        self.storage = kwargs.pop('storage')
-        self.indexer = kwargs.pop('indexer')
-        self.incoming = kwargs.pop('incoming')
+        self.chef = kwargs.pop('chef')
         super(TestingApp, self).__init__(*args, **kwargs)
         # Setup Keystone auth_token fake cache
         self.token = self.VALID_TOKEN
@@ -129,9 +126,9 @@ class TestingApp(webtest.TestApp):
         elif self.auth_mode == "remoteuser":
             req.remote_user = self.user
         response = super(TestingApp, self).do_request(req, *args, **kwargs)
-        metrics = tests_utils.list_all_incoming_metrics(self.incoming)
-        self.storage.process_new_measures(
-            self.indexer, self.incoming, metrics, sync=True)
+        for sack in self.chef.incoming.iter_sacks():
+            self.chef.process_new_measures_for_sack(
+                sack, blocking=True, sync=True)
         return response
 
 
@@ -178,9 +175,7 @@ class RestTest(tests_base.TestCase, testscenarios.TestWithScenarios):
 
         self.app = TestingApp(app.load_app(conf=self.conf,
                                            not_implemented_middleware=False),
-                              storage=self.storage,
-                              indexer=self.index,
-                              incoming=self.incoming,
+                              chef=self.chef,
                               auth_mode=self.auth_mode)
 
     def _fake_lazy_load(self, name):
@@ -222,7 +217,7 @@ class RootTest(RestTest):
             r = self.app.get("/")
         self.assertEqual(
             json.loads(r.text)['build'],
-            pbr.version.VersionInfo('gnocchi').version_string())
+            gnocchi.__version__)
 
     def test_status(self):
         with self.app.use_admin_user():
@@ -1944,3 +1939,66 @@ class QueryStringSearchAttrFilterTest(tests_base.TestCase):
                           ]},
                           {"=": {"foo": "quote"}},
                       ]})
+
+
+class TestMeasureQuery(tests_base.TestCase):
+    def test_equal(self):
+        q = api.SearchMetricController.MeasureQuery({"=": 4})
+        self.assertTrue(q(4))
+        self.assertFalse(q(40))
+
+    def test_gt(self):
+        q = api.SearchMetricController.MeasureQuery({">": 4})
+        self.assertTrue(q(40))
+        self.assertFalse(q(4))
+
+    def test_and(self):
+        q = api.SearchMetricController.MeasureQuery(
+            {"and": [{">": 4}, {"<": 10}]})
+        self.assertTrue(q(5))
+        self.assertFalse(q(40))
+        self.assertFalse(q(1))
+
+    def test_or(self):
+        q = api.SearchMetricController.MeasureQuery(
+            {"or": [{"=": 4}, {"=": 10}]})
+        self.assertTrue(q(4))
+        self.assertTrue(q(10))
+        self.assertFalse(q(-1))
+
+    def test_modulo(self):
+        q = api.SearchMetricController.MeasureQuery(
+            {"=": [{"%": 5}, 0]})
+        self.assertTrue(q(5))
+        self.assertTrue(q(10))
+        self.assertFalse(q(-1))
+        self.assertFalse(q(6))
+
+    def test_math(self):
+        q = api.SearchMetricController.MeasureQuery(
+            {
+                u"and": [
+                    # v+5 is bigger 0
+                    {u"≥": [{u"+": 5}, 0]},
+                    # v-6 is not 5
+                    {u"≠": [5, {u"-": 6}]},
+                ],
+            }
+        )
+        self.assertTrue(q(5))
+        self.assertTrue(q(10))
+        self.assertFalse(q(11))
+
+    def test_empty(self):
+        q = api.SearchMetricController.MeasureQuery({})
+        self.assertFalse(q(5))
+        self.assertFalse(q(10))
+
+    def test_bad_format(self):
+        self.assertRaises(api.SearchMetricController.MeasureQuery.InvalidQuery,
+                          api.SearchMetricController.MeasureQuery,
+                          {"foo": [{"=": 4}, {"=": 10}]})
+
+        self.assertRaises(api.SearchMetricController.MeasureQuery.InvalidQuery,
+                          api.SearchMetricController.MeasureQuery,
+                          {"=": [1, 2, 3]})
